import { world, system, EnchantmentTypes } from "@minecraft/server";
import { itemRegister } from "./itemRegister";
import { itemInfo } from "./itemHandler";

const cooldowns = new Map();

export function startCooldown(player, key, duration) {
    const endTime = Date.now() + duration * 1000;
    const playerCooldowns = cooldowns.get(player.id) || {};
    playerCooldowns[key] = endTime;
    cooldowns.set(player.id, playerCooldowns);
}

export function isOnCooldown(player, key) {
    const playerCooldowns = cooldowns.get(player.id) || {};
    const endTime = playerCooldowns[key];
    if (endTime) {
        return Date.now() < endTime;
    }
    return false;
}

export let playerdata = {};
const Factor = 565;
const defaultDuration = 0.25;
const unicodes = [
    "", "", "", "",
    "", "", "", "",
    "", "", "", "", 
    "", "", "", "",
    "", "",
];
function updateUnicodePercent(percent) {
    const index = Math.floor((percent / 100) * unicodes.length) - 1;
    return unicodes[index] ?? unicodes[3];
}

function spawnCriticalHitParticles(dimension, location) {
    const particlePos = { x: location.x, y: location.y + 0.625, z: location.z };
    for (let i = 0; i < 3; i++) {
        dimension.spawnParticle("dave:critical_hit_custom", particlePos);
    }
}

function handleCooldowns(player, item, duration, type) {
    const cooldownKey = `cd:${type}Timer`;

    startCooldown(player, cooldownKey, duration);
    player.setDynamicProperty(cooldownKey, 0);

    let timer = 0;
    const runId = system.runInterval(() => {
        timer++;
        const timerInSecond = timer / 20;

        if (timerInSecond <= duration) {
            const percent = (timerInSecond / duration) * 100;
            player.onScreenDisplay.setActionBar(`${updateUnicodePercent(percent)}`);
            player.addTag(`cd:has${type}Cooldown`);

            const damagePercentages = [0.2, 0.33, 0.36, 0.43, 0.54, 0.6, 0.7, 0.8, 0.92, 1];
            const damageIndex = Math.floor(percent / 10);
            const damageMultiplier = damagePercentages[damageIndex] ?? 1;

            itemInfo.forEach(e => {
                if (e.typeId === item.typeId) {
                    player.setDynamicProperty(`cd:${type}Damage`, e.damage * damageMultiplier);
                }
            });
            
        } else {
            player.setDynamicProperty(cooldownKey, 0);
            player.onScreenDisplay.setActionBar(` `);
            player.removeTag(`cd:has${type}Cooldown`);
            system.clearRun(runId);
        }
    }, 1);
}

function activateCooldown(player) {
    const inv = player.getComponent("inventory")?.container;
    const item = inv?.getItem(player.selectedSlotIndex);

    if (player.hasTag("hand")) {
        handleCooldowns(player, item, defaultDuration, "sweep");
        return;
    }

    if (item) {
        let itemFound = false;

        for (const reg of itemRegister) {
            if (item.typeId.includes(reg.item)) {
                itemFound = true;
                for (const element of reg.specific) {
                    if (element.type === "all" || 
                        (Array.isArray(element.type) && element.type.some(type => item.typeId.includes(type)))) 
					{
                        handleCooldowns(player, item, element.duration, "sweep");
                    }
                }
                break;
            }
        }

        if (!itemFound) {
            handleCooldowns(player, item, defaultDuration, "sweep");
        }
    }
}

world.afterEvents.entityHitEntity.subscribe(event => {
    const player = event.damagingEntity;
    const entity = event.hitEntity;

    if (player?.typeId !== "minecraft:player") return;

    const isFalling = player.isFalling;
    const isOnGround = player.isOnGround;
    const isInWater = player.isInWater;
    const dmgCooldown = player.getDynamicProperty('cd:damage') || 0; // Default damage if property is missing
    
    console.log("Player state: Falling:", isFalling, " OnGround:", isOnGround, " InWater:", isInWater);
    
    // Simplified critical hit check
    if (isFalling && !isOnGround && !isInWater) {
        console.log("Critical hit conditions met");

        entity.applyDamage(dmgCooldown * 1.5);
        const location = entity.location;

        if (location) {
            console.log("Playing critical hit sound at:", location);
            world.playSound("critical_hit", location, { pitch: 1.0, volume: 1.0 });  // Test sound
            spawnCriticalHitParticles(world.getDimension(player.dimension.id), location);
        } else {
            console.error("Invalid location for sound playback.");
        }
    }

    // Activate cooldown after processing hit
    activateCooldown(player);
});





world.afterEvents.entityHitBlock.subscribe(({ damagingEntity }) => {
    const player = damagingEntity;

    if (player?.typeId === "minecraft:player") {
        activateCooldown(player);
    }
});

system.runInterval(() => {
    world.getPlayers().forEach(player => {
        const inv = player.getComponent("inventory")?.container;
        const mainHandItem = inv?.getItem(player.selectedSlotIndex);

        if (!mainHandItem || (!mainHandItem?.typeId.includes("axe") &&
            !mainHandItem?.typeId.includes("sword") &&
            !mainHandItem?.typeId.includes("mace") &&
            !mainHandItem?.typeId.includes("trident") &&
            !mainHandItem?.typeId.includes("shovel") &&
            !mainHandItem?.typeId.includes("hoe"))) {
            player.addTag("hand");
        } else {
            player.removeTag("hand");
        }
    });
}, 10);

system.runInterval(() => {
    world.getPlayers().forEach(player => {
        if (player.hasTag("trigger_damage_cooldown")) {
            activateCooldown(player);

            player.removeTag("trigger_damage_cooldown");
        }
    });
}, 20);
